APP_NAME = "xappt"

PLUGIN_TYPE_TOOL = 1
PLUGIN_TYPE_INTERFACE = 2

PLUGIN_PREFIX = "xappt-", "xappt_"

PLUGIN_PATH_ENV = "XAPPT_PLUGIN_PATH"
INTERFACE_ENV = "XAPPT_INTERFACE"
INTERFACE_DEFAULT = "stdio"

TRUTHY_STRINGS = ("yes", "y", "true")
